package de.hft.stuttgart.citydoctor2.webservice.database;

import java.time.LocalDateTime;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;

import de.hft.stuttgart.citydoctor2.webservice.model.entities.Token;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.User;

public class TokenQueries {

	private TokenQueries() {
		// not used
	}

	public static Token getTokenForTokenString(String inputToken, String address, EntityManager manager) {
		manager.getTransaction().begin();

		Token token = null;
		try {
			TypedQuery<Token> q = manager.createQuery("FROM Token t WHERE t.tokenString = ?1 AND address = ?2", Token.class);
			q.setParameter(1, inputToken);
			q.setParameter(2, address);
			token = q.getSingleResult();
		} catch (NoResultException e) {
			// ignore
		} finally {
			manager.getTransaction().commit();
		}
		return token;
	}
	
	public static void persistToken(Token token, EntityManager manager) {
		manager.getTransaction().begin();
		try {
			manager.persist(token);
		} finally {
			manager.getTransaction().commit();
		}
	}
	
	public static void updateExpirationDate(LocalDateTime expirationDate, Token token, EntityManager manager) {
		manager.getTransaction().begin();
		try {
			token.setExpires(expirationDate);
		} finally {
			manager.getTransaction().commit();
		}
	}

	public static Token getTokenForUser(User user, String addr, EntityManager manager) {
		manager.getTransaction().begin();

		Token token = null;
		try {
			TypedQuery<Token> q = manager.createQuery("FROM Token t WHERE t.user.id = ?1 AND address = ?2", Token.class);
			q.setParameter(1, user.getId());
			q.setParameter(2, addr);
			token = q.getSingleResult();
		} catch (NoResultException e) {
			// ignore
		} finally {
			manager.getTransaction().commit();
		}
		return token;
	}

}
